from collections.abc import Iterable
from typing import Any, TypeGuard


def is_iterable(
    obj: Any, base_type: tuple[type, ...] = (str, bytes)
) -> TypeGuard[Iterable]:
    return isinstance(obj, Iterable) and not isinstance(obj, base_type)



from collections.abc import Iterable


def strip_comments(text: str, comments: str = "#") -> Iterable[str]:
    for line in text.strip().splitlines():
        stripped: str = line.split(comments, 1)[0].strip()
        if stripped:
            yield stripped



from . import once
from ._init import DEFAULT_FILTER, init_logging
from ._timer import Timer, timer
from .once import (
    critical_once,
    debug_once,
    error_once,
    info_once,
    log_once,
    success_once,
    trace_once,
    warning_once,
)

__all__ = [
    "DEFAULT_FILTER",
    "Timer",
    "critical_once",
    "debug_once",
    "error_once",
    "info_once",
    "init_logging",
    "log_once",
    "once",
    "success_once",
    "timer",
    "trace_once",
    "warning_once",
]



import functools

from loguru import logger


@functools.cache
def log_once(level: int | str, message: str, *args, depth: int = 1, **kwargs) -> None:
    logger.opt(depth=depth).log(level, message, *args, **kwargs)


trace_once = functools.partial(log_once, level="TRACE")
debug_once = functools.partial(log_once, level="DEBUG")
info_once = functools.partial(log_once, level="INFO")
success_once = functools.partial(log_once, level="SUCCESS")
warning_once = functools.partial(log_once, level="WARNING")
error_once = functools.partial(log_once, level="ERROR")
critical_once = functools.partial(log_once, level="CRITICAL")



import logging
import sys

import rich.traceback
from loguru import logger

from ._handler import InterceptHandler

DEFAULT_FILTER: dict[str | None, str | int | bool] = {
    "everett": logging.INFO,
    "git.cmd": logging.INFO,
    "jax._src": logging.INFO,
    "numba.core": logging.INFO,
    "urllib3.connectionpool": logging.INFO,
}


def init_logging(level: str | int = logging.NOTSET) -> None:
    rich.traceback.install(show_locals=True)
    logger.remove()
    logger.add(sys.stderr, level=level, filter=DEFAULT_FILTER)
    logging.basicConfig(handlers=[InterceptHandler()], level=0, force=True)



from collections.abc import Sequence
from typing import Any, TypeGuard


def is_sequence(
    obj: Any, base_type: tuple[type, ...] = (str, bytes)
) -> TypeGuard[Sequence]:
    return isinstance(obj, Sequence) and not isinstance(obj, base_type)



import os
from typing import Any, TypeVar

import pydantic

import avocado as ac

_C = TypeVar("_C", bound=pydantic.BaseModel)


def load_pydantic(
    fpath: str | os.PathLike[str], cls: type[_C], *, ext: str | None = None
) -> _C:
    data: Any = ac.deserialize(fpath, ext=ext)
    return cls.model_validate(data)


def save_pydantic(
    fpath: str | os.PathLike[str], data: pydantic.BaseModel, *, ext: str | None = None
) -> None:
    ac.serialize(fpath, data.model_dump(), ext=ext)



import os
from collections.abc import Callable
from pathlib import Path
from typing import Any

import avocado as ac

_READERS: dict[str, Callable[..., Any]] = {
    ".json": ac.load_json,
    ".toml": ac.load_toml,
    ".yaml": ac.load_yaml,
    ".yml": ac.load_yaml,
}


_WRITERS: dict[str, Callable[..., None]] = {
    ".json": ac.save_json,
    ".toml": ac.save_toml,
    ".yaml": ac.save_yaml,
    ".yml": ac.save_yaml,
}


def serialize(
    fpath: str | os.PathLike[str], data: Any, *, ext: str | None = None
) -> None:
    fpath: Path = Path(fpath)
    if ext is None:
        ext = fpath.suffix
    if ext not in _WRITERS:
        msg: str = f"Unsupported file extension: {ext}"
        raise ValueError(msg)
    writer = _WRITERS[ext]
    fpath.parent.mkdir(parents=True, exist_ok=True)
    writer(fpath, data)


def deserialize(fpath: str | os.PathLike[str], *, ext: str | None = None) -> Any:
    fpath: Path = Path(fpath)
    if ext is None:
        ext = fpath.suffix
    if ext not in _READERS:
        msg: str = f"Unsupported file extension: {ext}"
        raise ValueError(msg)
    reader = _READERS[ext]
    return reader(fpath)



import functools
import time
import types
from collections.abc import Callable
from typing import ParamSpec, TypeVar

from loguru import logger

_P = ParamSpec("_P")
_T = TypeVar("_T")


class Timer:
    _end: float
    _start: float
    depth: int | None = None
    level: int | str
    name: str | None

    def __init__(
        self,
        name: str | None = None,
        *,
        depth: int | None = None,
        level: int | str = "DEBUG",
    ) -> None:
        self.name = name
        self.depth = depth
        self.level = level

    def __call__(self, fn: Callable[_P, _T]) -> Callable[_P, _T]:
        if self.name is None:
            self.name = fn.__name__ + "()"
        if self.depth is None:
            self.depth = 3

        @functools.wraps(fn)
        def wrapped(*args: _P.args, **kwargs: _P.kwargs) -> _T:
            with self:
                result: _T = fn(*args, **kwargs)
            return result

        return wrapped

    def __enter__(self) -> None:
        self.start()

    def __exit__(
        self,
        exc_type: type[BaseException] | None,
        exc_value: BaseException | None,
        traceback: types.TracebackType | None,
    ) -> None:
        self.stop()

    @property
    def elapsed(self) -> float:
        return self._end - self._start

    def start(self) -> None:
        self._start = time.perf_counter()

    def stop(self) -> None:
        self._end = time.perf_counter()
        logger.opt(depth=self.depth or 2).log(
            self.level, "{} executed in {} sec.", self.name or "Block", self.elapsed
        )


def timer(
    name: str | None = None,
    *,
    depth: int | None = None,
    level: int | str = "DEBUG",
) -> Timer:
    return Timer(name=name, depth=depth, level=level)



from . import logging, serde, text, typing
from .logging import (
    DEFAULT_FILTER,
    Timer,
    critical_once,
    debug_once,
    error_once,
    info_once,
    init_logging,
    log_once,
    success_once,
    timer,
    trace_once,
    warning_once,
)
from .serde import (
    deserialize,
    load_json,
    load_pydantic,
    load_toml,
    load_yaml,
    save_json,
    save_pydantic,
    save_toml,
    save_yaml,
    serialize,
)
from .text import strip_comments
from .typing import StrPath, is_iterable, is_sequence

__all__ = [
    "DEFAULT_FILTER",
    "StrPath",
    "Timer",
    "critical_once",
    "debug_once",
    "deserialize",
    "error_once",
    "info_once",
    "init_logging",
    "is_iterable",
    "is_sequence",
    "load_json",
    "load_pydantic",
    "load_toml",
    "load_yaml",
    "log_once",
    "logging",
    "save_json",
    "save_pydantic",
    "save_toml",
    "save_yaml",
    "serde",
    "serialize",
    "strip_comments",
    "success_once",
    "text",
    "timer",
    "trace_once",
    "typing",
    "warning_once",
]
